/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */
package org.gvsig.mobile.extension.sensor.dialog;

import java.net.URL;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

import org.gvsig.i18n.Messages;
import org.gvsig.mobile.core.extensions.interfaces.ITask;
import org.gvsig.mobile.core.interfaces.TaskStatusListener;
import org.gvsig.mobile.sensor.SensorLocator;
import org.gvsig.mobile.sensor.SensorManager;
import org.gvsig.mobile.sensor.SensorService;
import org.gvsig.mobile.ui.lw.AppSettings;
import org.gvsig.mobile.ui.lw.DialogConstants;
import org.gvsig.mobile.ui.lw.MessageBox;
import org.gvsig.mobile.ui.lw.OkCancelDialog;
import org.osgi.framework.BundleContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.zaval.lw.LwAList;
import org.zaval.lw.LwBorderLayout;
import org.zaval.lw.LwButton;
import org.zaval.lw.LwCombo;
import org.zaval.lw.LwCompList;
import org.zaval.lw.LwConstraints;
import org.zaval.lw.LwGridLayout;
import org.zaval.lw.LwLabel;
import org.zaval.lw.LwPanel;
import org.zaval.lw.LwScrollPan;
import org.zaval.lw.LwTextField;
import org.zaval.lw.LwToolkit;

/**
 * @author <a href="mailto:csanchez@prodevelop.es"> Carlos Snchez Perin </a>
 *                  <a href="www.prodevelop.es">Prodevelop S.L. </a>
 */
public class AddSensorDialog extends OkCancelDialog implements TaskStatusListener{

	private LwCombo sensorsCombo;
	private LwCombo serversCombo;
	private LwCombo SRSCombo;
	private LwTextField serverText;
	private LwButton serverButton;
	private LwTextField SOSMetadata;
	private HashMap srsMap = new HashMap();
	private int selectedSRS = 0;
	private LwTextField TimeText;
	private LwButton runButton;
	private boolean run=false;
	private SensorManager manager;
	private SensorService service;
	
	private static final Logger logger = LoggerFactory.getLogger(AddSensorDialog.class);

	public AddSensorDialog(BundleContext bc) {
		super(false, bc);
	}
	
	public LwPanel buildMainPanel() {
		final LwPanel resp = new LwPanel();
		resp.setLwLayout(new LwBorderLayout());

		final LwConstraints l = DialogConstants.getTextFieldConstraints();
		final LwConstraints l1 = DialogConstants.getLabelConstraints();

		/**
		 * The top panel contains on WEST the labels and on CENTER the
		 * textfields It's on the NORTH of the back panel
		 */
		final LwPanel top = new LwPanel();
		top.setLwLayout(new LwBorderLayout());

		/**
		 * The labels
		 */
		final LwPanel left = new LwPanel();
		left.setLwLayout(new LwGridLayout(4, 1));

		left.add(l1, DialogConstants.getDefaultLabel(Messages
				.getText("SOSDialog.2")));
		left.add(l1, DialogConstants.getDefaultLabel(Messages
				.getText("SOSDialog.3")));
		left.add(l1, DialogConstants.getDefaultLabel(Messages
				.getText("SOSDialog.4")));
		left.add(l1, DialogConstants.getDefaultLabel(Messages
				.getText("SOSDialog.5")));

		/**
		 * The textfields, filling the panel horizontally
		 */
		final LwPanel right = new LwPanel();
		right.setLwLayout(new LwGridLayout(4, 1, LwToolkit.HORIZONTAL));

		sensorsCombo = getSensorsCombo();
		right.add(l, sensorsCombo);

		serversCombo = getServersCombo();
		right.add(l, serversCombo);

		/**
		 * Textfield + Button
		 */
		LwPanel composite = new LwPanel();
		composite.setLwLayout(new LwBorderLayout());

		serverText = getServerText();
		serverText.setText("");
		composite.add(LwBorderLayout.CENTER, serverText);

		serverButton = getServerButton();
		composite.add(LwBorderLayout.EAST, serverButton);

		right.add(l, composite);

		composite = new LwPanel();
		composite.setLwLayout(new LwBorderLayout());

		TimeText = getTimeText();
		TimeText.setText("");
		composite.add(LwBorderLayout.CENTER, TimeText);

		runButton = getRunButton();
		composite.add(LwBorderLayout.EAST, runButton);

		right.add(l, composite);

//		SRSCombo = getSRSCombo();
//		right.add(l, SRSCombo);

		top.add(LwBorderLayout.WEST, left);
		top.add(LwBorderLayout.CENTER, right);

		/**
		 * An empty Messages Text Box
		 */
		this.SOSMetadata = new LwTextField(); 
		this.SOSMetadata.getViewMan(true).setBorder("br.etched");
		this.SOSMetadata.setEditable(false);
		/**
		 * The tree must be scrollable. It's located on the CENTER of the back
		 * panel
		 */
		LwScrollPan t = new LwScrollPan(this.SOSMetadata);

		resp.add(LwBorderLayout.NORTH, top);
		resp.add(LwBorderLayout.CENTER, t);
		return resp;
	}

	/**
	 * fills the SRS combo
	 * 
	 * @param SRS
	 * @param add
	 */
	private void fillSRS(final Vector SRS, final boolean add) {
		int length = SRS.size();

		// inicializamos el combo y recuperamos la lista
		LwCompList list;
		if (SRSCombo == null) {
			this.SRSCombo = new LwCombo();
			list = new LwCompList();
		} else {
			list = (LwCompList) this.SRSCombo.getList();
		}

		selectedSRS = list.getSelectedIndex();

		// eliminamos SRS repetidos
		Vector cSRS = new Vector();
		String s;
		for (int i = 0; i < length; i++) {
			s = SRS.elementAt(i).toString();
			if (s != null && !cSRS.contains(s)) {
				cSRS.add(s);
			}
		}

		// En un Hashmap nos guardamos los SRS y las repeticiones
		String srs;
		length = cSRS.size();
		for (int i = 0; i < length; i++) {
			try {
				srs = cSRS.elementAt(i).toString();

			} catch (Exception ignore) {
				ignore.printStackTrace();
			}
		}

		// recorremos el hashmap y llenamos el combo
		Iterator it = srsMap.keySet().iterator();

		list.removeAll();
		while (it.hasNext()) {
			srs = it.next().toString();
			if (srs != null) {
				list.addItem(DialogConstants.getDefaultLabel(srs));
			}
		}

		if (list.count() == 0)
			list.addItem(DialogConstants.getDefaultLabel(""));
		SRSCombo.setList(list);

		if (selectedSRS < 0)
			selectedSRS = 0;

		SRSCombo.getList().select(selectedSRS);
	}

	private void clearSRS() {
		LwCompList list;
		if (SRSCombo == null) {
			this.SRSCombo = new LwCombo();
			list = new LwCompList();
		} else {
			list = (LwCompList) this.SRSCombo.getList();
		}
		list.removeAll();
		srsMap.clear();
	}
	
	private LwCombo getSensorsCombo() {
		final LwCombo resp = new LwCombo();
		LwCompList l = new LwCompList();
		l.addSelectionListener(this);
		this.loadSensorsFromPersistence(l);
		resp.setList(l);
		return resp;
	}
	
	private LwCombo getServersCombo() {
		final LwCombo resp = new LwCombo();
		LwCompList l = new LwCompList();
		// l.addItem(DialogConstants.getDefaultLabel(""));
		l.addSelectionListener(this);
		this.loadFromPersistence(l);
		resp.setList(l);
		return resp;
	}
	
	private LwCombo getSRSCombo() {
		final LwCombo resp = new LwCombo();
		LwCompList l = new LwCompList();
		l.addItem(DialogConstants.getDefaultLabel(""));
		resp.setList(l);
		return resp;
	}

	private LwButton getServerButton() {
		return DialogConstants.getButton("add", "add", this);
	}
	
	private LwButton getRunButton() {
			return DialogConstants.getButton("stop", "start", this);
	}

	private LwTextField getServerText() {
		return DialogConstants.getDefaultTextField();
	}
	
	private LwTextField getTimeText() {
		return DialogConstants.getDefaultTextField();
	}

	private void loadFromPersistence(LwAList l) {

		String[] sos = AppSettings.sosServers;

		l.removeAll();
		
		if (sos != null) {
			for (int i = 0; i < sos.length; i++) {
				if (sos[i].length() > 0)
					l.addItem(DialogConstants.getDefaultLabel(sos[i]));
			}
		}
	}

	private void loadSensorsFromPersistence(LwAList l) {
		//TODO cargar los sensores desde el SensorCache
		String[] sensors = {"gps"};
		l.removeAll();
		if (sensors != null) {
			for (int i = 0; i < sensors.length; i++) {
				if (sensors[i].length() > 0)
					l.addItem(DialogConstants.getDefaultLabel(sensors[i]));
			}
		}
	}
	
	public boolean isValidSRS(String SRS) {
		return false;
	}

	public void actionPerformed(Object arg0, Object arg1) {
				
		if (arg0 == this.getOkButton()) {
			close();
		} else if (arg0 == this.getCancelButton()) {
			close();
		} else if (arg0 == this.serverButton) {
			
		}else if (arg0 == this.runButton) {
			try {
				if(run){
					run=false;
					if (service!=null){
						service.stop();
						SOSMetadata.setText(SOSMetadata.getText()+"Stoped "+((LwLabel) sensorsCombo.getList().getSelected()).getText()+" Observation Insertion. \n");
					}
				}else{
					run=true;
					manager = SensorLocator.getManager();
					logger.warn("Sensor Manager : "+manager.toString());
					service = manager.getSensorService("urn:ogc:object:sensor:location:"+getSensor(), new URL(getServerUrl()));
					if (service!=null)
						logger.warn("Exist Service to Sensor : "+getSensor());
					service.start(new Double(TimeText.getText()).doubleValue());	//insertObservation every half second.
					SOSMetadata.setText(SOSMetadata.getText()+"Started "+((LwLabel) sensorsCombo.getList().getSelected()).getText()+" Observation Insertion. \n");
				}
			} catch (Exception e) {
				run=false;
				MessageBox.showDialog(false, bc, Messages.getText("Unable_to_connect_to_server") + ". " + e.getMessage(),null,MessageBox.MESSAGE_TYPE_ERROR, MessageBox.MESSAGE_ONLY_OK );
				logger.error(e.getMessage());
				logger.error(e.fillInStackTrace().toString());
			}    	
		}
	}

	
	/**
	 * This method is called after the dialog is closed.
	 * 
	 * @return the url of the server selected by the user.
	 */
	public String getServerUrl() {
		String url = null;
		try {
			Object selected = serversCombo.getList().getSelected();

			if (selected != null) {
				url = ((LwLabel) selected).getText();
			}
		} catch (Exception e) {
			// Ignore
		}
		return url;		
	}
	
	/**
	 * This method is called after the dialog is closed.
	 * 
	 * @return the url of the server selected by the user.
	 */
	public String getSensor() {
		String sensor=null;
		try {
			Object selected = sensorsCombo.getList().getSelected();

			if (selected != null) {
				sensor = ((LwLabel) selected).getText();
			}
		} catch (Exception e) {
			// Ignore
		}
		return sensor;		
	}

	/**
	 * This method is called after the dialog is closed.
	 * 
	 * @return the SRS selected by the user.
	 */
	public String getSrs() {
		return ((LwLabel) SRSCombo.getList().getSelected()).getText();
	}

	public String getTitle() {
		// TODO: Localize this
		return Messages.getText("SOSDialog.1");
		// return "Sensor Dialog";
	}
	
	public void taskFinished(ITask task) {}

	public void taskInited(ITask task) {}

	public void taskProgress(ITask task, int progress) {}
}